##
## Module to fetch_stat.pl from the Linksys SLM 2005/2008
## Untested after rewritten as a module since my switch is dead
## 
## Written by Joakim Andersson, joakim at iqcc dot se
##

use strict;
use LWP::UserAgent;
use Net::HTTP;

our $modver  = '1.00 (2009-04-12)';
our $modname = "linksys_slm200x.pm";
our $modcreator = "Joakim Andersson";

#######################################################
# Shared variables

our $SET;			# Settings
our $SYSTEM;		# System-info

# Creating UserAgent-object
my $ua = LWP::UserAgent->new();
$ua->agent("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

##
## Poll port
##

my %poll_port_cache = ();

sub switch_poll_port
{
	my $portnum = shift;
	my $is_first_poll = shift || 0;
	
	my $logtag = "$modname: poll_port:";
	
	##
	## Is it a port or LAG?
	##
	
	$portnum =~ /^(T)?(\d+)$/ || &log_n_die("$logtag Invalid portnum $portnum!");
	my $is_trunk = $1;
	my $id = $2 - 1;
	
	if ($is_trunk)
	{
		&log("Sorry, don't know yet how to poll a trunk :-(", 1);
		return {};
	}
	
	if ($is_first_poll)
	{
		##
		## We get all ports on the same time, so we cache it
		##
		
		my $res = $ua->get("http://$SET->{'SWITCH'}/stat?page=Overview&submit=Refresh&");
		&log_n_die("$logtag Unable to fetch /stat - ".$res->status_line) unless ($res->is_success);
		
		my $data = $res->content;
		my $title = '';
		$title = $1 if ($data =~ /<title>([^<]+)<\/title>/);
		
		if ($data =~ /Forbidden Access/ || $title =~ /(Duplicate Administrator|Login)/)
		{
			&log("Session invalid", $SET->{'VERBOSE'});
			
			$SYSTEM->{'cookie'} = '';
			return 0;
		}
		
		&log("Fetched /stat, got title $title", $SET->{'VERBOSE'} >= 3);
		
		#numPorts = 8;
		#txByteCnt = new Array("699281","135923212","3824036007","1471631562","1725075107","0","1061477462","0");
		#txFrmCnt  = new Array("6363","856402","3178053","6522413","6912960","0","4743757","0");
		#rxByteCnt = new Array("10530","940535956","964557023","4112346940","79522740","0","205741272","0");
		#rxFrmCnt  = new Array("130","12474445","6068274","5115395","5610528","0","3207577","0");
		#txErrCnt  = new Array("0","0","0","0","0","0","0","0");
		#rxErrCnt  = new Array("0","0","61","0","1","0","0","0");
		#txBcastCnt = new Array("6223","4011680","268700","4683461","726567","0","4706107","0");
		#rxBcastCnt = new Array("5","696488","4142","24686","3980810","0","3177007","0");
		
		%poll_port_cache = ();
		foreach (split(/\n/, $data))
		{
			next unless (/^\s*([^ ]+)\s*=\s*new\s*Array\("(.+)"\)/);
			my @array = split('","', $2);
			$poll_port_cache{$1} = \@array;
		}
	}
	
	##
	## Parses the data, is stored in javascript-variables in the beginning of the page
	##
	
	my %vars = ();
	$vars{'goodOctetSND'}	= $poll_port_cache{'txByteCnt'}->[$id];
	$vars{'txGood'}			= $poll_port_cache{'txFrmCnt'}->[$id];
	$vars{'txBroadcast'}	= $poll_port_cache{'txBcastCnt'}->[$id];
	#$vars{'txMulticast'}	= '';
	$vars{'txDrop'}			= $poll_port_cache{'txErrCnt'}->[$id];
	
	$vars{'goodOctetRCV'}	= $poll_port_cache{'rxByteCnt'}->[$id];
	$vars{'rxGood'}			= $poll_port_cache{'rxFrmCnt'}->[$id];
	$vars{'rxBroadcast'}	= $poll_port_cache{'rxBcastCnt'}->[$id];
	#$vars{'rxMulticast'}	= '';
	$vars{'rxFCS'}			= $poll_port_cache{'rxErrCnt'}->[$id];
	
	return \%vars;
}

##
## Resets stats for all ports
## Not implemented for the 2748, seems to be too much headache
##

sub switch_clear_stat
{
	my $logtag = "$modname: clear_stat:";
	
	my $res = $ua->get("http://$SET->{'SWITCH'}/stat?page=Overview&submit=Clear");
	&log_n_die("$logtag Unable to fetch /stat?page=Overview&submit=Clear - ".$res->status_line) unless ($res->is_success);
	
	return 1;		# 302 = Probably OK
}

##
## Reading systeminfo that doesn't change (often)
##

sub switch_system_info
{
	my $logtag = "$modname: system_info:";

	my $res = $ua->get("http://$SET->{'SWITCH'}/sys?submit=Refresh");
	&log_n_die("Unable to fetch /sys - ".$res->status_line) unless ($res->is_success);
	
	my $data = $res->content;
	my $title = '';
	$title = $1 if ($data =~ /<title>([^<]+)<\/title>/);
	
	if ($data =~ /Forbidden Access/ || $title =~ /(Duplicate Administrator|Login)/)
	{
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	&log("Fetched /sys, got title $title", $SET->{'VERBOSE'} >= 4);
	
	##
	## Finds the info we needs
	##
	
	my $row = 0;
	my %vars = ();
	
	my $fw_on_next = 0;
	
	foreach (split(/\n+/, $data))
	{
		$row++;
		#print "RAD $row: $_\n";
		
		if (/<input[^>]*name="([^"]+)"[^>]*value="([^"]+)"/i)
		{
			$vars{$1} = $2;
			#print "  $1 -> $2\n";
		}
		
		#<td class="style1" width="190">MAC Address</td>
		#<td class="style1" width="230">00-1e-e5-00-ee-f2</td>
		#<td class="style1">S/W Version</td>
		#<td class="style1">1.0.1</td>
		
		if (/S\/W Version/i)
		{
			$fw_on_next = 1;
		}
		elsif ($fw_on_next)
		{
			$SYSTEM->{'firmware'} = $1 if (/>([\d\.a-fA-F]+)</);
			$fw_on_next = 0;
		}
	}
	
	##
	## For compability with the Dell output
	##
	
	$SYSTEM->{'model'}      = $vars{'productIdentificationDisplayName'};
	$SYSTEM->{'model'}      =~ s/&nbsp;/ /g;
	
	$SYSTEM->{'macaddress'} = uc $1 if ($data =~ />([a-f0-9\-]{17})</);
	$SYSTEM->{'macaddress'} =~ tr/-/:/;
	
	$SYSTEM->{'servicetag'} = '';
	$SYSTEM->{'assettag'}   = '';
	$SYSTEM->{'sysname'}    = $vars{'nm'};
	$SYSTEM->{'location'}   = $vars{'sl'};
	$SYSTEM->{'serial'}     = '';
	
	$SYSTEM->{'timeout'}	= $vars{'timeout'};
	
	# Not available?
	$SYSTEM->{'uptime'} = 0;

	##
	## Parses info about every port
	## Seperate page
	##
	
	$res = $ua->get("http://$SET->{'SWITCH'}/ports?submit=Refresh");
	&log_n_die("Unable to fetch /ports - ".$res->status_line) unless ($res->is_success);
	
	$data = $res->content;
	$title = '';
	$title = $1 if ($data =~ /<title>([^<]+)<\/title>/);
	
	if ($data =~ /Forbidden Access/ || $title =~ /(Duplicate Administrator|Login)/)
	{
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	# Tries to get model from title
	$SYSTEM->{'model'} = $1 if ($title =~ /^(Linksys SLM[^ ]+) /i);
	
	&log("Fetched /ports, got title $title", $SET->{'VERBOSE'} >= 4);
	
	my @curSpeed = ();
	#var curSpeed = new Array("  Down   ","  100FDX ","  Down   ","  100FDX "," 1000FDX ","  Down   ","  100FDX ","  Down   ");
	#numPorts = 8;
	
	if ($data =~ /curSpeed\s*=\s*new\s+Array\s*\("(.+)"\)/)
	{
		@curSpeed = split('","', $1);
	}
	else
	{
		&log_n_die("$logtag Unable to locate curSpeed variable in /ports!");
	}
	
	$SYSTEM->{'numberOfPorts'} = $1 if ($data =~ /numPorts\s*=\s*(\d+);/);
	&log_n_die("Unable to locate variable numPorts in /ports!") unless ($SYSTEM->{'numberOfPorts'});
	$SYSTEM->{'numberOfTrunks'} = 0;
	
	my $tot = $SYSTEM->{'numberOfPorts'} + $SYSTEM->{'numberOfTrunks'};
	my %ports = ();
	
	foreach my $num (1 .. $tot)
	{
		my $trunknum = $num - $SYSTEM->{'numberOfPorts'};
		my $portnum = $num;
		$portnum = "T$trunknum" if ($trunknum > 0);	# It's a trunk
		
		my $index = $num - 1;
		
		my $link = lc $curSpeed[$index] || 'down';
		$link =~ tr/ //d;
		$link =~ s/fdx/\/Full/;
		$link =~ s/hdx/\/Half/;
		
		$ports{$portnum} = {};
		$ports{$portnum}->{'link'} = $link;
		$ports{$portnum}->{'is_active'} = ($link ne 'down');
		$ports{$portnum}->{'order'} = $index;
	}
	
	return \%ports;
}

##
## Login to setup the session
##

sub switch_login
{
	my $logtag = "$modname: login:";
	
	&log("Logging in into $SET->{'SWITCH'}...", $SET->{'VERBOSE'});
	
	my $res = $ua->get("http://$SET->{'SWITCH'}/pass?username=$SET->{'USERNAME'}&password=$SET->{'PASSWORD'}&submit=Apply");
	&log_n_die("$logtag Unable to fetch /pass - ".$res->status_line) unless ($res->is_success);
	
	my $data = $res->content;
	my $title = '';
	$title = $1 if ($data =~ /<title>([^<]*)<\/title>/);
	#print "$data\n";
	
	if ($data =~ /Forbidden Access/ || $title =~ /(Duplicate Administrator|Login)/)
	{
		if ($title =~ /Duplicate/)
		{
			# Sleeps until the other sessions times out, hopefully
			my $timeout = 10;	#int(($system{'timeout'} || 300) / 2);
			&log("Another session is active, sleeps $timeout secs...", $SET->{'VERBOSE'});
			sleep $timeout;
			
			# Tries again 
			return &switch_login;
		}
		
		&log_n_die("$logtag Unable to login, got '$title' - ".$res->status_line) unless ($res->is_success);
	}
	
	# No cookie, the switch tracks ip
	return 'logged_in';
}

##
## Logout - Not implemented/needed
##

sub switch_logout
{
	my $logtag = "$modname: logout:";
	
	my $res = $ua->get("http://$SET->{'SWITCH'}/logout?submit=Apply");
	&log_n_die("$logtag Unable to fetch /logout - ".$res->status_line) unless ($res->is_success);
	
	my $data = $res->content;
	my $title = '';
	$title = $1 if ($data =~ /<title>([^<]*)<\/title>/);
	#print "$data\n";
	
	$SYSTEM->{'cookie'} = '';
	&log("Logged out, got title $title", $SET->{'VERBOSE'} >= 4);
	
	return 1;
}

###########################################################

&log("$modname $modver loaded", $SET->{'VERBOSE'});

1;

